==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/Metadata.java <==
package de.fzj.unicore.ucc.actions;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Map;

import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.io.FileUtils;
import org.json.JSONObject;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.MetadataClient;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.TaskClient;
import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.JSONUtil;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;

/**
 * Metadata operations on a storage
 * 
 * @author schuller
 */
public class Metadata extends ActionBase {

    public static String OPT_COMMAND_LONG = "command";
    public static String OPT_COMMAND = "C";
    public static String OPT_STORAGE_LONG = "storage";
    public static String OPT_STORAGE = "s";
    public static String OPT_META_LONG = "metadata-service";
    public static String OPT_META = "m";
    public static String OPT_FILE_LONG = "file";
    public static String OPT_FILE = "f";
    public static String OPT_QUERY_LONG = "query";
    public static String OPT_QUERY = "q";
    public static String OPT_QUERYADV_LONG = "advanced-query";
    public static String OPT_QUERYADV = "a";
    protected String metaURL;
    protected MetadataClient meta;
    protected String command;
    protected String path;
    protected File file;
    protected String query;
    boolean advanced;

    public String getName() {
        return "metadata";
    }

    @Override
    @SuppressWarnings("all")
    protected void createOptions() {
        super.createOptions();

        getOptions().addOption(OptionBuilder.withLongOpt(OPT_COMMAND_LONG).
                withDescription("Metadata command: create, read, update, delete, start-extract, search").
                isRequired(true).
                hasArg().
                create(OPT_COMMAND));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_STORAGE_LONG).
                withDescription("Storage address").
                isRequired(false).
                withArgName("Storage").
                hasArg().
                create(OPT_STORAGE));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_META_LONG).
                withDescription("Metadata service address").
                isRequired(false).
                withArgName("MetadataService").
                hasArg().
                create(OPT_META));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_FILE_LONG).
                withDescription("File containing metadata").
                isRequired(false).
                withArgName("Filename").
                hasArg().
                create(OPT_FILE));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_QUERY_LONG).
                withDescription("Query string for search").
                isRequired(false).
                withArgName("Query").
                hasArg().
                create(OPT_QUERY));

        getOptions().addOption(OptionBuilder.withLongOpt(OPT_QUERYADV_LONG).withDescription("Advanced query").isRequired(false).create(OPT_QUERYADV));
    }

    @Override
    public void process() {
        super.process();
        command = getOption(OPT_COMMAND_LONG, OPT_COMMAND);
        verbose("Operation = " + command);
        
        //figure out which metadata service to use
        metaURL = getOption(OPT_META_LONG, OPT_META);
        
        if (metaURL!= null) {
        
            EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
            epr.addNewAddress().setStringValue(metaURL);
            try {
                String dn = findServerName(epr);
                if (dn != null) {
                    WSUtilities.addServerIdentity(epr, dn);
                }
                meta = new MetadataClient(epr, securityProperties);
            } catch (Exception ex) {
                error("Can't create metadata client for " + metaURL, ex);
                endProcessing(ERROR_CLIENT);
            }
        } else {
        	String storage = getOption(OPT_STORAGE_LONG, OPT_STORAGE);
            if(storage==null){
            	String desc=getArgument(1);
            	//try to interpret path as a full UNICORE storage path
            	Location loc=new Location(desc, registry, securityProperties, this);
            	storage=loc.getSmsEpr();
            	path=loc.getName();
            }
            findMetadataService(storage);
        }

        
        if (metaURL == null) {
            message("Cannot find a metadata service to use!");
            printUsage();
            endProcessing(ERROR_CLIENT);
        }
        
        verbose("Using metadata service at " + metaURL);

        String fName = getOption(OPT_FILE_LONG, OPT_FILE);
        if (fName != null) {
            file = new File(fName);
        }

        if ("read".startsWith(command)) {
            if(path==null)path = getArgument(1);
            doGet();
        } else if ("create".startsWith(command)) {
        	if(path==null)path = getArgument(1);
            doCreate();
        } else if ("update".startsWith(command)) {
        	if(path==null)path = getArgument(1);
            doUpdate();
        } else if ("delete".startsWith(command)) {
        	if(path==null)path = getArgument(1);
            doDelete();
        } else if ("start-extract".startsWith(command)) {
            path = getArgument(1);
            //TODO depth

            doStartExtract();
        } else if ("search".startsWith(command)) {
            query = getOption(OPT_QUERY_LONG, OPT_QUERY);
            if (query == null) {
                error("Please provide a query string!", null);
                endProcessing(1);
            }
            advanced = getBooleanOption(OPT_QUERYADV_LONG, OPT_QUERYADV);
            doSearch();
        } else {
            error("Unknown command : " + command, null);
            endProcessing(1);
        }

    }

    protected void doGet() {
        try {
            Map<String, String> result = meta.getMetadata(path);
            String json = JSONUtil.asJSON(result).toString(2);
            message(json);
            if (file != null) {
                FileUtils.writeStringToFile(file, json);
            }
            lastMeta = result;
        } catch (Exception ex) {
            error("Error getting metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doCreate() {
        try {
            Map<String, String> data = readData();
            meta.createMetadata(path, data);
            lastMeta = data;
        } catch (Exception ex) {
            error("Error creating metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doUpdate() {
        try {
            Map<String, String> data = readData();
            meta.updateMetadata(path, data);
            lastMeta = data;
        } catch (Exception ex) {
            error("Error updating metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doSearch() {
        try {
            Collection<String> results = meta.search(query, advanced);
            for (String s : results) {
                message(s);
            }
        } catch (Exception ex) {
            error("Error searching metadata", ex);
            endProcessing(1);
        }
    }

    protected void doDelete() {
        try {
            meta.deleteMetadata(path);
        } catch (Exception ex) {
            error("Error deleting metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doStartExtract() {
        try {
            int depth = 10;
            TaskClient task = meta.startMetadataExtraction(path, depth);
            message("Extraction started, task address = " + task.getUrl());
        } catch (Exception ex) {
            error("Error starting metadata extraction for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    /**
     * read metadata from file or stdin
     * @throws Exception
     */
    protected Map<String, String> readData() throws Exception {
        String json = null;
        if (file != null) {
            verbose("Reading data from the file: " + file.getName());
            json = FileUtils.readFileToString(file);
        } else {
            //read from stdin
            verbose("Reading from stdIn (it does not work from whitin the shell)");

            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            String str = "";
            StringBuilder buffer = new StringBuilder();
            do {
                System.out.print(">> ");
                str = in.readLine();
                if (str != null) {
                    buffer.append(str);
                }
            } while (str != null && !str.isEmpty());

            json = buffer.toString();
        }
        JSONObject j = new JSONObject(json);
        return JSONUtil.asMap(j);
    }

    protected void findMetadataService(String storage) {
        Location td = null;
        StorageClient sms = null;
        try {
            td = new Location(storage, registry, securityProperties, this);
            EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
            epr.addNewAddress().setStringValue(td.getSmsEpr());
            sms = new StorageClient(td.getSmsEpr(), epr, securityProperties);
            sms.getCurrentTime();
            verbose("Have storage epr=" + td.getSmsEpr() + ", access OK.");
        } catch (Exception e) {
            error("Can't access :" + storage, e);
            endProcessing(ERROR);
        }

        try {
            if (!sms.supportsMetadata()) {
                message("Storage does not support metadata.");
                endProcessing(ERROR);
            }
            meta = sms.getMetadataClient();
            metaURL = meta.getUrl();
        } catch (EndProcessingException epe) {
            throw epe;
        } catch (Exception e) {
            error("Can't get metadata service for :" + storage, e);
            endProcessing(ERROR);
        }
    }

    @Override
    public String getDescription() {
        return "Perform operations on metadata";
    }

    @Override
    public String getSynopsis() {
        return "Performs operations on metadata. "
                + "Either a storage or directly a metadata service can be given.";
    }

    @Override
    public String getArgumentList() {
        return "[resource-name]";
    }

    @Override
    public String getCommandGroup() {
        return "Data management";
    }

    private String getArgument(final int argN) {
        try {
            return getCommandLine().getArgs()[argN];
        } catch (IndexOutOfBoundsException ex) {
            error("This method requires at least " + argN + " arguments", null);
            endProcessing();
        }
        return null;

    }

    /**
     * for unit tests
     * @return the result of the last operation
     */
    public static Map<String, String> getLastMeta() {
        return lastMeta;
    }
    private static Map<String, String> lastMeta;
}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/Metadata.java <==
package de.fzj.unicore.ucc.actions;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Map;

import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.io.FileUtils;
import org.json.JSONObject;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.MetadataClient;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.TaskClient;
import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.JSONUtil;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;

/**
 * Metadata operations on a storage
 * 
 * @author schuller
 */
public class Metadata extends ActionBase {

    public static String OPT_COMMAND_LONG = "command";
    public static String OPT_COMMAND = "C";
    public static String OPT_STORAGE_LONG = "storage";
    public static String OPT_STORAGE = "s";
    public static String OPT_META_LONG = "metadata-service";
    public static String OPT_META = "m";
    public static String OPT_FILE_LONG = "file";
    public static String OPT_FILE = "f";
    public static String OPT_QUERY_LONG = "query";
    public static String OPT_QUERY = "q";
    public static String OPT_QUERYADV_LONG = "advanced-query";
    public static String OPT_QUERYADV = "a";
    protected String metaURL;
    protected MetadataClient meta;
    protected String command;
    protected String path;
    protected File file;
    protected String query;
    boolean advanced;

    public String getName() {
        return "metadata";
    }

    @Override
    @SuppressWarnings("all")
    protected void createOptions() {
        super.createOptions();

        getOptions().addOption(OptionBuilder.withLongOpt(OPT_COMMAND_LONG).
                withDescription("Metadata command: create, read, update, delete, start-extract, search").
                isRequired(true).
                hasArg().
                create(OPT_COMMAND));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_STORAGE_LONG).
                withDescription("Storage address").
                isRequired(false).
                withArgName("Storage").
                hasArg().
                create(OPT_STORAGE));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_META_LONG).
                withDescription("Metadata service address").
                isRequired(false).
                withArgName("MetadataService").
                hasArg().
                create(OPT_META));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_FILE_LONG).
                withDescription("File containing metadata").
                isRequired(false).
                withArgName("Filename").
                hasArg().
                create(OPT_FILE));
        getOptions().addOption(OptionBuilder.withLongOpt(OPT_QUERY_LONG).
                withDescription("Query string for search").
                isRequired(false).
                withArgName("Query").
                hasArg().
                create(OPT_QUERY));

        getOptions().addOption(OptionBuilder.withLongOpt(OPT_QUERYADV_LONG).withDescription("Advanced query").isRequired(false).create(OPT_QUERYADV));
    }

    @Override
    public void process() {
        super.process();
        command = getOption(OPT_COMMAND_LONG, OPT_COMMAND);
        verbose("Operation = " + command);
        
        //figure out which metadata service to use
        metaURL = getOption(OPT_META_LONG, OPT_META);
        
        if (metaURL!= null) {
        
            EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
            epr.addNewAddress().setStringValue(metaURL);
            try {
                String dn = findServerName(epr);
                if (dn != null) {
                    WSUtilities.addServerIdentity(epr, dn);
                }
                meta = new MetadataClient(epr, securityProperties);
            } catch (Exception ex) {
                error("Can't create metadata client for " + metaURL, ex);
                endProcessing(ERROR_CLIENT);
            }
        } else {
        	String storage = getOption(OPT_STORAGE_LONG, OPT_STORAGE);
            if(storage==null){
            	String desc=getArgument(1);
            	//try to interpret path as a full UNICORE storage path
            	Location loc=new Location(desc, registry, securityProperties, this);
            	storage=loc.getSmsEpr();
            	path=loc.getName();
            }
            findMetadataService(storage);
        }

        
        if (metaURL == null) {
            message("Cannot find a metadata service to use!");
            printUsage();
            endProcessing(ERROR_CLIENT);
        }
        
        verbose("Using metadata service at " + metaURL);

        String fName = getOption(OPT_FILE_LONG, OPT_FILE);
        if (fName != null) {
            file = new File(fName);
        }

        if ("read".startsWith(command)) {
            if(path==null)path = getArgument(1);
            doGet();
        } else if ("create".startsWith(command)) {
        	if(path==null)path = getArgument(1);
            doCreate();
        } else if ("update".startsWith(command)) {
        	if(path==null)path = getArgument(1);
            doUpdate();
        } else if ("delete".startsWith(command)) {
        	if(path==null)path = getArgument(1);
            doDelete();
        } else if ("start-extract".startsWith(command)) {
            path = getArgument(1);
            //TODO depth

            doStartExtract();
        } else if ("search".startsWith(command)) {
            query = getOption(OPT_QUERY_LONG, OPT_QUERY);
            if (query == null) {
                error("Please provide a query string!", null);
                endProcessing(1);
            }
            advanced = getBooleanOption(OPT_QUERYADV_LONG, OPT_QUERYADV);
            doSearch();
        } else {
            error("Unknown command : " + command, null);
            endProcessing(1);
        }

    }

    protected void doGet() {
        try {
            Map<String, String> result = meta.getMetadata(path);
            String json = JSONUtil.asJSON(result).toString(2);
            message(json);
            if (file != null) {
                FileUtils.writeStringToFile(file, json);
            }
            lastMeta = result;
        } catch (Exception ex) {
            error("Error getting metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doCreate() {
        try {
            Map<String, String> data = readData();
            meta.createMetadata(path, data);
            lastMeta = data;
        } catch (Exception ex) {
            error("Error creating metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doUpdate() {
        try {
            Map<String, String> data = readData();
            meta.updateMetadata(path, data);
            lastMeta = data;
        } catch (Exception ex) {
            error("Error updating metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doSearch() {
        try {
            Collection<String> results = meta.search(query, advanced);
            for (String s : results) {
                message(s);
            }
        } catch (Exception ex) {
            error("Error searching metadata", ex);
            endProcessing(1);
        }
    }

    protected void doDelete() {
        try {
            meta.deleteMetadata(path);
        } catch (Exception ex) {
            error("Error deleting metadata for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    protected void doStartExtract() {
        try {
            int depth = 10;
            TaskClient task = meta.startMetadataExtraction(path, depth);
            message("Extraction started, task address = " + task.getUrl());
        } catch (Exception ex) {
            error("Error starting metadata extraction for <" + path + ">", ex);
            endProcessing(1);
        }
    }

    /**
     * read metadata from file or stdin
     * @throws Exception
     */
    protected Map<String, String> readData() throws Exception {
        String json = null;
        if (file != null) {
            verbose("Reading data from the file: " + file.getName());
            json = FileUtils.readFileToString(file);
        } else {
            //read from stdin
            verbose("Reading from stdIn (it does not work from within the shell)");

            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            String str = "";
            StringBuilder buffer = new StringBuilder();
            do {
                System.out.print(">> ");
                str = in.readLine();
                if (str != null) {
                    buffer.append(str);
                }
            } while (str != null && !str.isEmpty());

            json = buffer.toString();
        }
        JSONObject j = new JSONObject(json);
        return JSONUtil.asMap(j);
    }

    protected void findMetadataService(String storage) {
        Location td = null;
        StorageClient sms = null;
        try {
            td = new Location(storage, registry, securityProperties, this);
            EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
            epr.addNewAddress().setStringValue(td.getSmsEpr());
            sms = new StorageClient(td.getSmsEpr(), epr, securityProperties);
            sms.getCurrentTime();
            verbose("Have storage epr=" + td.getSmsEpr() + ", access OK.");
        } catch (Exception e) {
            error("Can't access :" + storage, e);
            endProcessing(ERROR);
        }

        try {
            if (!sms.supportsMetadata()) {
                message("Storage does not support metadata.");
                endProcessing(ERROR);
            }
            meta = sms.getMetadataClient();
            metaURL = meta.getUrl();
        } catch (EndProcessingException epe) {
            throw epe;
        } catch (Exception e) {
            error("Can't get metadata service for :" + storage, e);
            endProcessing(ERROR);
        }
    }

    @Override
    public String getDescription() {
        return "perform operations on metadata";
    }

    @Override
    public String getSynopsis() {
        return "Performs operations on metadata. "
                + "Either a storage or directly a metadata service can be given.";
    }

    @Override
    public String getArgumentList() {
        return "[resource-name]";
    }

    @Override
    public String getCommandGroup() {
        return "Data management";
    }

    private String getArgument(final int argN) {
        try {
            return getCommandLine().getArgs()[argN];
        } catch (IndexOutOfBoundsException ex) {
            error("This method requires at least " + argN + " arguments", null);
            endProcessing();
        }
        return null;

    }

    /**
     * for unit tests
     * @return the result of the last operation
     */
    public static Map<String, String> getLastMeta() {
        return lastMeta;
    }
    private static Map<String, String> lastMeta;
}
